# Root Saga Patterns

A root Saga aggregates multiple Sagas to a single entry point for the SagaMiddleware to run.

In the [beginner tutorial](../introduction/BeginnerTutorial.md), it is shown that your root saga will look something like this:

```dart
rootSaga() sync* {
  yield All({
    #hello: helloSaga(),
    #watch: watchIncrementAsync(),
  });
  // code after all-effect
}
```

This is one of a few ways to implement your root. Here, the `All` effect is used with an array and your sagas will be executed in parallel. Other root implementations may help you better handle errors and more complex data flow.

## Non-blocking fork effects

To start, there is one popular implementation that behaves similarly to the tutorial root saga behavior:

```dart
rootSaga() sync* {
  yield Fork(saga1);
  yield Fork(saga2);
  yield Fork(saga3);
  // code after fork-effect
}
```

Using three unique `yield Fork` will give back a task descriptor three times. The resulting behavior in your app is that all of your sub-sagas are started and executed in the same order. Since `Fork` is non-blocking, the `rootSaga` can finish while the child sagas continue to run and be blocked by their internal effects.

The difference between one big All effect and several Fork effects is that the `All` effect is blocking, so *code after all-effect* (see comments in above code) is executed when all children sagas complete, while `Fork` effects are non-blocking so *code after Fork-effect* is executed immediately after yielding the Fork effects. Another difference is that you can get task descriptors when using Fork effects, so in the subsequent code you can Cancel/Join the forked task via task descriptors.

## Nesting Fork effects in All effect

```dart
rootSaga() sync* {
  var tasks = AllResult();
  yield All({
    #saga1: Fork(saga1),
    #saga2: Fork(saga2),
    #saga3: Fork(saga3),
  });
}
```

There is another popular pattern when designing root saga: nesting `Fork` effects in an `All` effect. By doing so, you can get an array of task descriptors, and the code after the `All` effect will be executed immediately because each `Fork` effect is non-blocking and synchronously returning a task descriptor.

Note that though `Fork` effects are nested in an `All` effect, they are always connected to the parent task through the underlying ForkQueue. Uncaught errors from forked tasks bubble to the parent task and thus abort it (and all its child tasks) - they cannot be caught by the parent task.

## Avoid nesting fork effects in race effect

```dart
  // DO NOT DO THIS. The fork effect always win the race immediately.
  yield Race({
    #fork: Fork(someSaga),
    #take: Take(pattern: SomeAction),
    #future: someFuture,
  });
```

On the other hand, `Fork` effects in a `Race` effect is most likely a bug. In the above code, since `Fork` effects are non-blocking, they will always win the race immediately.

## Keeping the root alive

In practice, these implementations aren't terribly practical because your `rootSaga` will terminate on the first error in any individual child effect or saga and crash your whole app! Ajax requests in particular would put your app at the mercy of the status of any endpoints your app makes requests to.

[`Spawn`](https://pub.dev/documentation/redux_saga/latest/redux_saga/Spawn-class.html) is an effect that will *disconnect* your child saga from its parent, allowing it to fail without crashing its parent. Obviously, this does not relieve us from our responsibility as developers to still handle errors as they arise. In fact, it's possible that this might obscure certain failures from the developer's viewpoint and cause problems further down the road.

```dart
rootSaga() sync* {
  yield Spawn(saga1);
  yield Spawn(saga2);
  yield Spawn(saga3);
}
```

In this implementation, even if one saga were to fail, the `rootSaga` and other sagas will not be killed. However, this can also be problematic since the failing saga would be unavailable for the app's lifetime.

## Keeping everything alive

In some cases, it may be desirable for your sagas to be able to restart in the event of failure. The benefit is that your app and sagas may continue to work after failing, i.e. a saga that `yield TakeEvery(pattern: myActionType)`. But we do not recommend this as a blanket solution to keep all sagas alive. It is very likely that it makes more sense to let your saga fail in sanely and predictably and handle/log your error.

For example, Keeping your saga alive would cause more problems than it solves:

```dart
sagaThatMayCrash () sync* {
  // wait for something that happens _during app startup_
  yield Take(pattern: AppInit);

  // assume it dies here
  yield Call(doSomethingThatMayCrash);
}
```

> If the sagaThatMayCrash is restarted, it will restart and wait for an action that only happens once when the application starts up. In this scenario, it restarts, but it never recovers.

But for the specific situations that would benefit from starting.

```dart
rootSaga() sync* {
  var sagas = {
    #saga1: saga1,
    #saga2: saga2,
    #saga3: saga3,
  };

  yield All(sagas.map((key, saga) => MapEntry(key, Spawn(() sync* {
        while (true) {
          var success = false;
          yield Try(() sync* {
            yield Call(saga);
            success = true;
          }, Catch: (e, s) {
            print(e);
          });
          if (success) break;
        }
      }))));
}
```

This strategy maps our child sagas to spawned generators (detaching them from the root parent) which start our sagas as subtasks in a `Try` block. Our saga will run until termination, and then be automatically restarted. The `Catch` block harmlessly handles any error that may have been thrown by, and terminated, our saga.
